/*
  CardPlayerV3 Code
  By: Trevor Sherrard, Joseph Bunch, and Michael Vaught

   Uses MIFARE RFID card using RFID-RC522 reader
   Uses MFRC522 - Library
   -----------------------------------------
               MFRC522      Arduino
               Reader/PCD   Uno/101       
   Signal      Pin          Pin           
   -----------------------------------------
   RST/Reset   RST          9          
   SPI SS      SDA(SS)      10,8,7            
   SPI MOSI    MOSI         11 / ICSP-4
   SPI MISO    MISO         12 / ICSP-1
   SPI SCK     SCK          13 / ICSP-3
   Power       3.3V         3.3V

   -----------------------------------------
                            Arduino
               DFplayer     Uno/101       
   Signal      Pin          Pin           
   -----------------------------------------
   RX Pin      RX           5          
   TX Pin      TX           6       
   Power       VCC          5V

   -----------------------------------------
                            Arduino
               Button       Uno/101       
   Signal      Orientation  Pin           
   -----------------------------------------
   Button1     Left         3          
   Button2     Mid          2       
   Button3     Right        0

*/

#include <SPI.h>
#include <MFRC522.h>
#include "Arduino.h"
#include "SoftwareSerial.h"
#include "DFRobotDFPlayerMini.h"


#define RST_PIN         9   
#define SS1_PIN         10          
#define SS2_PIN         8
#define SS3_PIN         7

#define No_Readers 3
byte ssPins[] = {SS1_PIN, SS2_PIN, SS3_PIN}; 

//MFRC522 mfrc522(SS_PIN, RST_PIN);   // Create MFRC522 instance
// Create an MFRC522 instance :
MFRC522 mfrc522[No_Readers];
const String number[3];

SoftwareSerial mySoftwareSerial(6, 5); // RX,TX

DFRobotDFPlayerMini myDFPlayer;
void printDetail(uint8_t type, int value);
int button1Pin = 3; // left
int button2Pin = 2; // mid
int button3Pin = 0; // right
int button1Read;
int button2Read;
int button3Read;



//*****************************************************************************************//
void setup() {
  Serial.begin(115200);                                         // Initialize serial communications with the PC, COMMENT OUT IF IT FAILS TO PLAY WHEN DISCONNECTED FROM PC
  mySoftwareSerial.begin(9600);
  SPI.begin();                                                  // Init SPI bus
  pinMode(button1Pin,INPUT_PULLUP);                             // Init Buttons
  pinMode(button2Pin,INPUT_PULLUP);
  pinMode(button3Pin,INPUT_PULLUP);    
  number[0]="04";
  number[1]="02";
  number[2]="03";

  // Init RFID Card Readers
  for (uint8_t reader = 0; reader < No_Readers; reader++) {
    mfrc522[reader].PCD_Init(ssPins[reader], RST_PIN); // Init each MFRC522 card
    delay(100);
    Serial.print(F("Reader "));
    Serial.print(reader);
    Serial.print(F(": "));
    mfrc522[reader].PCD_DumpVersionToSerial();
  }

  Serial.println(F("Initializing DFPlayer ... (May take 3~5 seconds)"));

  if (myDFPlayer.begin(mySoftwareSerial,true,true)){
    Serial.println("DFPlayer Online :D");
    myDFPlayer.volume(30);  //Set volume value. From 0 to 30 
  }

  else{
    Serial.println("Connecting to DFPlayer Mini failed! :(");
  }

}



//*****************************************************************************************//
void loop() {



  // Prepare key - all keys are set to FFFFFFFFFFFFh at chip delivery from the factory.
  MFRC522::MIFARE_Key key;
  for (byte i = 0; i < 6; i++) key.keyByte[i] = 0xFF;

  //some variables we need
  byte block;
  byte len;
  MFRC522::StatusCode status;
  uint8_t pageAddr = 0x06;
  byte buffer[18];  //data transfer buffer (16+2 bytes data+CRC)
  byte size = sizeof(buffer);
  String number_new_cards = "";  


  //-------------------------------------------

  // Reset the loop if no new card present on the sensor/reader. This saves the entire process when idle.
  for (uint8_t i = 0; i < No_Readers; i++) {
  int command[3]; //change
  if ( mfrc522[i].PICC_IsNewCardPresent()) {

    // Select one of the cards
    if ( ! mfrc522[i].PICC_ReadCardSerial()) {
      return;
    }

    Serial.println(F("**Card Detected:**"));

    //-------------------------------------------
    MFRC522::Uid serial = mfrc522[i].uid;

    mfrc522[i].PICC_DumpDetailsToSerial(&serial); //dump some details about the card
    MFRC522::PICC_Type piccType = mfrc522[i].PICC_GetType(serial.sak);
    //mfrc522.PICC_DumpToSerial(&(mfrc522.uid));      //uncomment this to see all blocks in hex

    //-------------------------------------------

    Serial.print(F("Number: "));


    //---------------------------------------- GET NUMBER AND PLAY THE SONG

    byte buffer2[18];
    block = 1;
    len = 18;
    number[i] = "";
    number_new_cards = "";  

    //Check which type of card is being scanned. The below if statement and code is for the older cards.
    if (piccType == MFRC522::PICC_TYPE_MIFARE_1K){
      status = mfrc522[i].PCD_Authenticate(MFRC522::PICC_CMD_MF_AUTH_KEY_A, 1, &key, &(mfrc522[i].uid)); //line 834
      if (status != MFRC522::STATUS_OK) {
        Serial.print(F("Authentication failed: "));
        Serial.println(mfrc522[i].GetStatusCodeName(status));
        return;
      }

      status = mfrc522[i].MIFARE_Read(block, buffer2, &len);
      if (status != MFRC522::STATUS_OK) {
        Serial.print(F("Reading failed: "));
        Serial.println(mfrc522[i].GetStatusCodeName(status));
        return;
      }

      //Read the Number from the card
      for (uint8_t k = 0; k < 16; k++)
      {
        if ((char)buffer2[k] == NULL){
          number[i] += '0';
        }
        else{
          number[i] += (char)buffer2[k];
        }
      }
    }

    //For newer programmable cards
    else{
      status = mfrc522[i].MIFARE_Read(pageAddr, buffer, &size);
      if (status != MFRC522::STATUS_OK) {
        Serial.print(F("Reading failed: "));
        Serial.println(mfrc522[i].GetStatusCodeName(status));
        return;
      }

      //Dump a byte array to Serial
      for (byte i = 0; i < 16; i++) {
        if (buffer[i] == 0xFE){   // Break if end of number
          break;
        }

        //Adds hex number converted to character to number string
        else if (((char)buffer[i] != NULL ) && (char)buffer[i] != 'n'){
          number_new_cards += (char)buffer[i];
        }
      }
      number[i] = number_new_cards;
    }

    Serial.println(number[i].toInt()); //prints the read number
    command[i] = number[i].toInt();    //Saves number into command to be played
 


    //----------------------------------------

    Serial.println(F("\n**End Reading**\n"));
    mfrc522[i].PICC_HaltA();
    mfrc522[i].PCD_StopCrypto1();
  }



    //PLAY SONG
  // read the state of the pushbutton value
  button1Read = digitalRead(button1Pin);
  button2Read = digitalRead(button2Pin);
  button3Read = digitalRead(button3Pin);

  //check if the pushbutton is pressed. If it is, the buttonState is LOW:
  if (button1Read == LOW) {
    Serial.println("button1Read");
    myDFPlayer.play(command[0]);
    delay(500);
  } 
  if (button2Read == LOW) {
    Serial.println("button2Read");
    myDFPlayer.play(command[1]);
    delay(500);
  } 
  if (button3Read == LOW) {
    Serial.println("button3Read");
    myDFPlayer.play(command[2]);
    delay(500);
  } 
  }
}



//*****************************************************************************************//
